
var cube_prevonload=window.onload;

window.onload = cube_onload;


var usewidth=600;
var useheight=930;


console.log("3d1...");
import * as THREE from './three/build/three.module.js';
console.log("3d2...");
import { OBJLoader } from './three/examples/jsm/loaders/OBJLoader.js'
console.log("3d3...");


function cube_onload()
{
  if (cube_prevonload)
  {
    cube_prevonload();
  }

  var square=document.getElementById('offscreen-square');
  if (square)
  {
    usewidth=square.clientWidth;
    useheight=square.clientHeight;
  }

    usewidth=window.innerWidth;
    useheight=window.innerHeight;



  newRenderCanvas();

  renderScene();

  init();
  renderer.render(scene, camera);

}

var scene = null;
var camera = null;
var renderer = null;

var materialLines = null;
var materialFloor = null;
var materialFaces = null;
var sceneGroup = null;

function newRenderCanvas()
{
  scene = new THREE.Scene();
  camera = new THREE.PerspectiveCamera( 45, (1.0*usewidth)/useheight, 0.1, 1000 );
  renderer = new THREE.WebGLRenderer( { antialias: true } );
  renderer.setClearColor(0xffffff,1);
  renderer.setSize( usewidth, useheight , false);
  document.body.appendChild( renderer.domElement );
  materialLines = new THREE.LineBasicMaterial( {
	  color: 0x000000,
	  linewidth: 2,
	  linecap: 'round', //ignored by WebGLRenderer
	  linejoin:  'round' //ignored by WebGLRenderer
  } );
  materialFloor = new THREE.LineBasicMaterial( {
	  color: 0x000000 /* 0xa0a0a0 */,
	  linewidth: 2,
	  linecap: 'round', //ignored by WebGLRenderer
	  linejoin:  'round' //ignored by WebGLRenderer
  } );
  materialFaces = new THREE.MeshBasicMaterial( { color: 0xffffff, side: THREE.DoubleSide, transparent: true, opacity:0.75 } );
}



function addRectangle(group,v1,v2,v3,v4,material,showFaces)
{
  var geometry = new THREE.Geometry();
  geometry.vertices.push(new THREE.Vector3( v1[0], v1[1], v1[2] ));
  geometry.vertices.push(new THREE.Vector3( v2[0], v2[1], v2[2] ));
  geometry.vertices.push(new THREE.Vector3( v3[0], v3[1], v3[2] ));
  geometry.vertices.push(new THREE.Vector3( v4[0], v4[1], v4[2] ));
  geometry.vertices.push(new THREE.Vector3( v1[0], v1[1], v1[2] ));
  if (showFaces)
  {
    geometry.faces.push( new THREE.Face3( 0, 1, 2 ) );
    geometry.faces.push( new THREE.Face3( 2, 3, 4 ) );
/*
    geometry.computeFaceNormals();
    geometry.computeVertexNormals();
*/
    var meshfaces = new THREE.Mesh( geometry, materialFaces );
    group.add( meshfaces );
  }
  var meshlines = new THREE.Line( geometry, material );
  group.add( meshlines );
}


function addCircle(group,center,rx,ry,material,showFaces)
{
  var geometry = new THREE.Geometry();
  var i;
  var nrsteps=24;

  for (i=0;i<=nrsteps;i++)
  {
    var angle = (i*2.0*Math.PI)/nrsteps;
    var cs = Math.cos(angle);
    var sn = Math.sin(angle);
    var x = center[0]+rx[0]*cs+ry[0]*sn;
    var y = center[1]+rx[1]*cs+ry[1]*sn;
    var z = center[2]+rx[2]*cs+ry[2]*sn;
    geometry.vertices.push(new THREE.Vector3( x, y, z ));
  }
  if (showFaces)
  {
    for (i=1;i<nrsteps-1;i++)
    {
      geometry.faces.push( new THREE.Face3( 0, i, i+1 ) );
    }
    var meshfaces = new THREE.Mesh( geometry, materialFaces );
    group.add( meshfaces );
  }
  var meshlines = new THREE.Line( geometry, material );
  group.add( meshlines );
}


function addCylinder(group,center,rx,ry,height,material,showFaces)
{
  addCircle(group,center,rx,ry, material, false);

  var cross = [0,0,0];
  cross[0] = rx[1]*ry[2]-rx[2]*ry[1];
  cross[1] = rx[2]*ry[0]-rx[0]*ry[2];
  cross[2] = rx[0]*ry[1]-rx[1]*ry[0];

  var length = Math.sqrt(cross[0]*cross[0]+
                         cross[1]*cross[1]+
			 cross[2]*cross[2]);

  cross[0] /= length;
  cross[1] /= length;
  cross[2] /= length;

  cross[0] *= height;
  cross[1] *= height;
  cross[2] *= height;

  addCircle(group,[center[0]+cross[0],center[1]+cross[1],center[2]+cross[2]],
    rx,ry, material, false);

  
  addRectangle(group,[center[0]-rx[0],center[1]-rx[1],center[2]-rx[2]],
    [center[0]+rx[0],center[1]+rx[1],center[2]+rx[2]],
    [center[0]+cross[0]+rx[0],center[1]+cross[1]+rx[1],center[2]+cross[2]+rx[2]],
    [center[0]+cross[0]-rx[0],center[1]+cross[1]-rx[1],center[2]+cross[2]-rx[2]], 
    material, false);


  addRectangle(group,[center[0]-ry[0],center[1]-ry[1],center[2]-ry[2]],
    [center[0]+ry[0],center[1]+ry[1],center[2]+ry[2]],
    [center[0]+cross[0]+ry[0],center[1]+cross[1]+ry[1],center[2]+cross[2]+ry[2]],
    [center[0]+cross[0]-ry[0],center[1]+cross[1]-ry[1],center[2]+cross[2]-ry[2]], 
    material, false);
}







function init()
{
  document.addEventListener( 'mousedown', onDocumentMouseDown, false );
  document.addEventListener( 'mousemove', onDocumentMouseMove, false );
  document.addEventListener( 'mouseup', onDocumentMouseUp, false );


  document.addEventListener( 'touchstart', onDocumentTouchStart, false );
  document.addEventListener( 'touchmove', onDocumentTouchMove, false );
  document.addEventListener( 'touchend', onDocumentTouchEnd, false );


  document.addEventListener( 'keypress', onDocumentKeyPress, false );
}

var mousedownSet = false;
var mouseXOnMouseDown;
var mouseYOnMouseDown;
var mouseX;
var mouseY;
var targetRotationXOnMouseDown;
var targetRotationYOnMouseDown;
var targetPositionXOnMouseDown;
var targetPositionYOnMouseDown;
var targetPositionZOnMouseDown;
var mousemode=0;


function onDocumentMouseDown( event )
{
  event.preventDefault();


  mousemode=0;
  if (event.shiftKey)
  {
    mousemode=1;
  }

  if (event.ctrlKey)
  {
    mousemode=2;
  }

  mousedownSet = true;
  mouseXOnMouseDown = event.clientX;
  mouseYOnMouseDown = event.clientY;
  targetRotationXOnMouseDown = sceneGroup.rotation.y;
  targetRotationYOnMouseDown = sceneGroup.rotation.x;
  targetPositionXOnMouseDown = sceneGroup.position.x;
  targetPositionYOnMouseDown = sceneGroup.position.y;
  targetPositionZOnMouseDown = sceneGroup.position.z;
}

function onDocumentMouseMove( event )
{
  if (mousedownSet)
  {
    mouseX = event.clientX;
    mouseY = event.clientY;
    switch (mousemode)
    {
    case 0:
      sceneGroup.rotation.y = targetRotationXOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.02;
      sceneGroup.rotation.x = targetRotationYOnMouseDown + ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    case 1:
      sceneGroup.position.z = targetPositionZOnMouseDown + ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    case 2:
      sceneGroup.position.x = targetPositionXOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.02;
      sceneGroup.position.y = targetPositionYOnMouseDown - ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    }
    renderer.render(scene, camera);
  }
}

function onDocumentMouseUp( event )
{
  mousedownSet = false;
}




function onDocumentTouchStart( event )
{
  event.preventDefault();


  mousemode=0;
  if (event.shiftKey)
  {
    mousemode=1;
  }

  if (event.ctrlKey)
  {
    mousemode=2;
  }

  mousedownSet = true;
  mouseXOnMouseDown = event.touches[0].clientX;
  mouseYOnMouseDown = event.touches[0].clientY;
  targetRotationXOnMouseDown = sceneGroup.rotation.y;
  targetRotationYOnMouseDown = sceneGroup.rotation.x;
  targetPositionXOnMouseDown = sceneGroup.position.x;
  targetPositionYOnMouseDown = sceneGroup.position.y;
  targetPositionZOnMouseDown = sceneGroup.position.z;
}

function onDocumentTouchMove( event )
{
  if (mousedownSet)
  {
    mouseX = event.touches[0].clientX;
    mouseY = event.touches[0].clientY;
    switch (mousemode)
    {
    case 0:
      sceneGroup.rotation.y = targetRotationXOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.02;
      sceneGroup.rotation.x = targetRotationYOnMouseDown + ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    case 1:
      sceneGroup.position.z = targetPositionZOnMouseDown + ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    case 2:
      sceneGroup.position.x = targetPositionXOnMouseDown + ( mouseX - mouseXOnMouseDown ) * 0.02;
      sceneGroup.position.y = targetPositionYOnMouseDown - ( mouseY - mouseYOnMouseDown ) * 0.02;
      break;
    }
    renderer.render(scene, camera);
  }
}

function onDocumentTouchEnd( event )
{
  mousedownSet = false;
}















function onDocumentKeyPress( event )
{
  var chCode = ('charCode' in event) ? event.charCode : event.keyCode;
  if (chCode == "s".charCodeAt(0) || chCode == "S".charCodeAt(0))
  {

    if (sceneGroup != null)
    {
      scene.remove(sceneGroup);
      sceneGroup=null;
    }
    cubeh=3-cubeh;
    renderScene();
    renderer.render(scene, camera);
  }
}






